#if !defined(__SANGWUG_KERNEL_MATH_MATH_EXTENSION_H_)
#define __SANGWUG_KERNEL_MATH_MATH_EXTENSION_H_ 1


#include "swMathConst.h"


//-----------------------------------------------------------------------------------------
//

double logb(double base, double x);
double asinh(double x);
double acosh(double x);  //  x >= 1
double atanh(double x);  //  -1 < x < 1


//-----------------------------------------------------------------------------------------
//  struct swMathExt

struct swMathExt
{
/**@name Public Members Function */
//@{
public :
	///
	static bool is_zero(double x, double tol = swMathConst::EPS)
	{  return -tol <= x && x <= tol;  }
	static bool is_positive(double x, double tol = swMathConst::EPS)
	{  return x > tol;  }
	static bool is_negative(double x, double tol = swMathConst::EPS)
	{  return x < -tol;  }

	static bool is_equal(double x, double y, double tol = swMathConst::EPS)
	{  return is_zero(x - y, tol);  }

	///  (lower, upper)
	static bool is_bounded_oo(double x, double lower, double upper, double tol = swMathConst::EPS)
	{  return lower - tol < x && x < upper + tol;  }
	///  [lower, upper]
	static bool is_bounded_cc(double x, double lower, double upper, double tol = swMathConst::EPS)
	{  return lower - tol <= x && x <= upper + tol;  }
	///  (lower, upper]
	static bool is_bounded_oc(double x, double lower, double upper, double tol = swMathConst::EPS)
	{  return lower - tol < x && x <= upper + tol;  }
	///  [lower, upper)
	static bool is_bounded_co(double x, double lower, double upper, double tol = swMathConst::EPS)
	{  return lower - tol <= x && x < upper + tol;  }

	///
	static double to_rad(double deg)
	{  return deg * swMathConst::TO_RAD;  }
	static double to_deg(double rad)
	{  return rad * swMathConst::TO_DEG;  }

	///
	static bool is_integer(double x, double tol = swMathConst::EPS);
	static bool is_real(double x, double tol = swMathConst::EPS);

	static double mantissa(double x, double* pow = 0L);

	///
	static double sign(double x)
	{  return x >= 0.0 ? 1.0 : -1.0;  }
	static double saturate(double x)
	{
		if (x >= 1.0) return 1.0;
		else if (x <= -1.0) return -1.0;
		else return x;
	}

	///
	static double round(double x);
	static double round(double x, int decimalPlace);

	///  GCD  :  Greatest Common Divisor
	static unsigned long gcd(unsigned long lhs, unsigned long rhs);
	static double gcd(double lhs, double rhs, double tol = swMathConst::EPS);

	///  LCM  :  Least Common Multiple
	static unsigned long lcm(unsigned long lhs, unsigned long rhs);
	static double lcm(double lhs, double rhs, double tol = swMathConst::EPS);

	///
	static unsigned long factorial(unsigned long n);
	static double factorial(double n, double tol = swMathConst::EPS);
	static unsigned long permutation(unsigned long lhs, unsigned long rhs);
	static double permutation(double lhs, double rhs, double tol = swMathConst::EPS);
	static unsigned long binomial(unsigned long lhs, unsigned long rhs);
	static double binomial(double lhs, double rhs, double tol = swMathConst::EPS);
//@}
};


#endif  //  __SANGWUG_KERNEL_MATH_MATH_EXTENSION_H_
