#include "InclinedBar.h"
#include <cassert>
#include <cmath>


//---------------------------------------------------------------------------
// CInclinedBar

//  CInclinedBar 
CInclinedBar::CInclinedBar(double len, double vy)
: len_(len), vy_(vy)
{
	//  bar ̴ ̰  A ϰ ӵ ̾  
	assert(len_ > 0.0 && vy_ < 0.0);
}

//   A 鿡   ð 
double CInclinedBar::getElapsedTime() const
{
	return -len_ / vy_;
}

//  ð t  bar   踦 
double CInclinedBar::calcInclinedAngle(double t) const
{
	double YA = calcPosA(t);
	double XB = calcPosB(t);
	//   B ġ  
	assert(0.0 <= YA && YA <= len_ && 0.0 <= XB && XB <= len_);

	//  [ 12-56]   
	return ::atan2(YA, XB);
}

//  ð t   A   
double CInclinedBar::calcPosA(double t) const
{
	//  ð   ܻ ó
	if (t < 0.0) return len_;
	//  Ѽҿð Ŭ    ˷ش
	else if (t > getElapsedTime()) return 0.0;

	//  [ 12-54]  A 
	return len_ + vy_ * t;
}

//  ð t   B   
double CInclinedBar::calcPosB(double t) const
{
	//  ð   ܻ ó
	if (t < 0.0) return 0.0;
	//  Ѽҿð Ŭ    ˷ش
	else if (t > getElapsedTime()) return len_;

	//  [ 12-55]  B 
	return ::sqrt(len_ * len_ - ::pow(len_ + vy_ * t, 2.0));
}

//  ð t   B  ӵ 
double CInclinedBar::calcLinearVelB(double t) const
{
	//  ð   ܻ ó
	if (t < 0.0) return 0.0;
	//  Ѽҿð Ŭ  ܻ ó
	else if (t > getElapsedTime()) return 0.0;

	double theta = calcInclinedAngle(t);

	//  [ 12-58]  B  ӵ
	return -vy_ * ::tan(theta);
}

//  ð t   B  ӵ 
double CInclinedBar::calcLinearAccelB(double t) const
{
	//  ð   ܻ ó
	if (t < 0.0) return 0.0;
	//  Ѽҿð Ŭ  ܻ ó
	else if (t > getElapsedTime()) return 0.0;

	//
	double theta = calcInclinedAngle(t);
	double omega = calcAngularVel(t);
	double alpha = calcAngularAccel(t);

	//  [ 12-60]  B  ӵ
	return len_ * (alpha * ::sin(theta) - omega * omega * ::cos(theta));
}

//  ð t  bar ӵ 
double CInclinedBar::calcAngularVel(double t) const
{
	//  ð   ܻ ó
	if (t < 0.0) return 0.0;
	//  Ѽҿð Ŭ  ܻ ó
	else if (t > getElapsedTime()) return 0.0;

	double theta = calcInclinedAngle(t);

	//  [ 12-57]  ӵ
	return -vy_ / (len_ * ::cos(theta));
}

//  ð t  bar ӵ 
double CInclinedBar::calcAngularAccel(double t) const
{
	//  ð   ܻ ó
	if (t < 0.0) return 0.0;
	//  Ѽҿð Ŭ  ܻ ó
	else if (t > getElapsedTime()) return 0.0;

	double theta = calcInclinedAngle(t);
	double omega = calcAngularVel(t);

	//  [ 12-59]  ӵ
	return omega * omega * ::tan(theta);
}
