#include "PinnedBar.h"
#include "ODE.h"
#include <cassert>
#include <cmath>


//---------------------------------------------------------------------------
// 

void derivative(double t, double* theta, double* dtheta_dt)
{
	//  theta = [ theta(0) ] = [ theta ] : bar 
	//          [ theta(1) ]   [ omega ] : bar ӵ

	//  dtheta_dt = [ dtheta_dt(0) ] = [ omega ] : bar ӵ
	//              [ dtheta_dt(1) ]   [ alpha ] : bar ӵ

	dtheta_dt[0] = theta[1];

	//  [ 13-34]    
	dtheta_dt[1] = CPinnedBar::K1 * sin(theta[0]) + CPinnedBar::K2;
}


//---------------------------------------------------------------------------
// CPinnedBar

double CPinnedBar::K1;
double CPinnedBar::K2;

//  CPinnedBar 
CPinnedBar::CPinnedBar(double accelCart, double maxElapsedTime, unsigned long maxStep)
: accelCart_(accelCart), maxElapsedTime_(maxElapsedTime), maxStep_(maxStep),
  len_(0.5), m_(2.0), g_(9.80665), Ic_(m_ * len_ * len_ / 12.0)
  
{
	//  bar ̴ ̾  
	assert(len_ > 0.0);
	result_ = new double [(maxStep_ + 1) * 2];

	double aa = Ic_ + m_ * len_ * len_ / 4.0;
	double bb = -m_ * len_ / 2.0;
	K1 = bb * g_ / aa;
	K2 = bb * accelCart_ / aa;

	calcInclinedAngle();
}

//  CPinnedBar Ҹ
CPinnedBar::~CPinnedBar()
{
	delete [] result_;
}

//  bar   踦 
void CPinnedBar::calcInclinedAngle()
{
	result_[0] = 0.0;
	result_[1 * (maxStep_ + 1)] = 0.0;

	//  Runge-Kutta algorithm 
	RungeKutta(derivative, 2, 2, (double*)result_, 0.0, maxElapsedTime_, maxStep_);
}

//  bar   踦 ȯ
double CPinnedBar::getInclinedAngle(unsigned long step) const
{
	return 0 <= step && step <= maxStep_ ? result_[step] : 0.0;
}
